from .clients import GenieACSClient


class TasksAPI:
    def __init__(self, client: GenieACSClient):
        self.client = client

    async def list(self, query=None):
        return await self.client.get("/tasks", params=query or {})

    async def add(self, device_id: str, command: str, params=None):
        """Add a task to a device"""
        payload = {"command": command, "params": params or {}}
        return await self.client.post(f"/devices/{device_id}/tasks", json=payload)

    async def get(self, task_id: str):
        return await self.client.get(f"/tasks/{task_id}")

    async def delete(self, task_id: str):
        return await self.client.delete(f"/tasks/{task_id}")



from .clients import GenieACSClient


class ProvisionsAPI:
    def __init__(self, client: GenieACSClient):
        self.client = client

    async def list(self):
        return await self.client.get("/provisions")

    async def get(self, provision_name: str):
        return await self.client.get(f"/provisions/{provision_name}")

    async def create(self, name: str, script: str):
        """Create a provision with GenieACS script"""
        payload = {"script": script}
        return await self.client.post(f"/provisions/{name}", json=payload)

    async def delete(self, provision_name: str):
        return await self.client.delete(f"/provisions/{provision_name}")



from .clients import GenieACSClient


class PresetsAPI:
    def __init__(self, client: GenieACSClient):
        self.client = client

    async def list(self):
        return await self.client.get("/presets")

    async def get(self, preset_name: str):
        return await self.client.get(f"/presets/{preset_name}")

    async def create(self, name: str, data: dict):
        return await self.client.post(f"/presets/{name}", json=data)

    async def delete(self, preset_name: str):
        return await self.client.delete(f"/presets/{preset_name}")



import httpx
from urllib.parse import urljoin


class GenieACSClient:
    def __init__(self, base_url="http://localhost:7557", username=None, password=None):
        self.base_url = base_url.rstrip("/")
        self.auth = (username, password) if username and password else None
        self.client = httpx.AsyncClient(
		        auth=self.auth,
		        timeout=10.0,
		        limits=httpx.Limits(max_connections=10, max_keepalive_connections=5)
        )
    
    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.close()
    
    
    def _url(self, path):
        return urljoin(f"{self.base_url}/", path.lstrip("/"))


    async def close(self):
        await self.client.aclose()
    
    async def get(self, path, **kwargs):
        r = await self.client.get(self._url(path), **kwargs)
        r.raise_for_status()
        return r.json()
    
    
    async def post(self, path, json=None, **kwargs):
        r = await self.client.post(self._url(path), json=json, **kwargs)
        r.raise_for_status()
        return r.json()
    
    
    async def put(self, path, json=None, **kwargs):
        r = await self.client.put(self._url(path), json=json, **kwargs)
        r.raise_for_status()
        return r.json()
        
    
    async def delete(self, path, **kwargs):
        r = await self.client.delete(self._url(path), **kwargs)
        r.raise_for_status()
        return r.status_code == 204
    
    
    


from .clients import GenieACSClient


class DevicesAPI:
    def __init__(self, client: GenieACSClient):
        self.client = client

    async def list(self, query=None):
        """List all devices with optional query parameters"""
        return await self.client.get("/devices", params=query or {})

    async def get(self, device_id: str):
        """Get details of a single device"""
        return await self.client.get(f"/devices/{device_id}")

    async def refresh_object(self, device_id: str, object_name: str):
        """Refresh a device object"""
        return await self.client.post(f"/devices/{device_id}/objects/{object_name}/refresh")

    async def delete(self, device_id: str):
        """Delete a device"""
        return await self.client.delete(f"/devices/{device_id}")


from .clients import GenieACSClient
from .devices import DevicesAPI
from .tasks import TasksAPI
from .presets import PresetsAPI
from .provisions import ProvisionsAPI

__all__ = [
    "GenieACSClient",
    "DevicesAPI",
    "TasksAPI",
    "PresetsAPI",
    "ProvisionsAPI"
]
